(* -----------------------------------------------------------------------------
  Load Multiframe TIF File On Demand With Annotations : 1.0
  Copyright  2015              : Copyright Adirondack Software & Graphics
  Last Modification             : 06-02-2015
  Source File                   : Unit1.pas
  Purpose                       : Project Main Form
  Compiler                      : Delphi XE7
  Target Platform               : Win32/Win64
  ImageEnVersion                : 6.0.1
  Operating System              : Windows 8.1
  This file is copyright (C) W W Miller, 1986-2015.
  It may be used without restriction. This code distributed on an "AS IS"
  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
  --------------------------------------------------------------------------- *)
unit Unit1;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants,
  System.Classes, Vcl.Graphics, System.UITypes, Vcl.Controls, Vcl.Forms,
  Vcl.Dialogs, Vcl.ExtDlgs, Vcl.ComCtrls, imageenview, ieview, iemview,
  Vcl.StdCtrls, Vcl.ExtCtrls, Vcl.Buttons, ievect, hyiedefs, hyieutils,
  iegdiplus;

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    Open1: TButton;
    ImageEnMView1: TImageEnMView;
    StatusBar1: TStatusBar;
    OpenPictureDialog1: TOpenPictureDialog;
    ImageEnVect1: TImageEnVect;
    AddObjects1: TSpeedButton;
    SelectObjects1: TSpeedButton;
    SaveAs1: TButton;
    SavePictureDialog1: TSavePictureDialog;
    SaveAnnotations1: TButton;
    Clear1: TButton;
    Panel2: TPanel;
    Panel3: TPanel;
    Exit1: TButton;
    Panel4: TPanel;
    Exit2: TButton;
    AddFrame1: TButton;
    DeleteSelectedFrame1: TButton;
    procedure FormCreate(Sender: TObject);
    procedure Open1Click(Sender: TObject);
    procedure ImageEnMView1ImageSelect(Sender: TObject; idx: Integer);
    procedure AddObjects1Click(Sender: TObject);
    procedure SelectObjects1Click(Sender: TObject);
    procedure SaveAs1Click(Sender: TObject);
    procedure ImageEnVect1NewObject(Sender: TObject; hobj: Integer);
    procedure ImageEnVect1UserSelectObject(Sender: TObject; hobj: Integer);
    procedure SaveAnnotations1Click(Sender: TObject);
    procedure Clear1Click(Sender: TObject);
    procedure ImageEnVect1KeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure ImageEnVect1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure ImageEnVect1AfterDrawObject(Sender: TObject; hobj: Integer;
      destination: TIEBitmap; destCanvas: TIECanvas; destRect: TRect;
      drawingAlpha: Boolean; var handled: Boolean);
    procedure Exit1Click(Sender: TObject);
    procedure AddFrame1Click(Sender: TObject);
    procedure DeleteSelectedFrame1Click(Sender: TObject);
  private
    { Private declarations }
    procedure UpdateGUI;
    procedure SaveAnnotationsToImageEnMView;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

uses imageenio, iemio;

procedure TForm1.UpdateGUI;
begin
  SaveAs1.Enabled := ImageEnVect1.Bitmap.Modified;
  SaveAnnotations1.Enabled := ImageEnVect1.ObjectsCount <> 0;
  Clear1.Enabled := ImageEnMView1.ImageCount > 0;
end;

procedure TForm1.SaveAnnotationsToImageEnMView;
{ Display the annotations in TImageEnMView }
begin
  if ImageEnMView1.SelectedImage > -1 then
  begin
    ImageEnMView1.MIO.Params[ImageEnMView1.SelectedImage]
      .ImageEnAnnot.CopyFromTImageEnVect(ImageEnVect1);
    ImageEnMView1.UpdateImage(ImageEnMView1.SelectedImage);
  end;
end;

procedure TForm1.AddFrame1Click(Sender: TObject);
var
  i: Integer;
begin
  OpenPictureDialog1.Options := [ofHideReadOnly, ofAllowMultiSelect,
    ofEnableSizing];
  if OpenPictureDialog1.Execute then
  begin
    for i := 0 to OpenPictureDialog1.Files.Count - 1 do
      ImageEnMView1.MIO.LoadFromFile(OpenPictureDialog1.Files[i]);
  end;
end;

procedure TForm1.AddObjects1Click(Sender: TObject);
{ Set MouseInteractVt. }
begin
  if AddObjects1.Down then
    ImageEnVect1.MouseInteractVt := [miPutEllipse]
  else
    ImageEnVect1.MouseInteractVt := [];
end;

procedure TForm1.Clear1Click(Sender: TObject);
{ Clear all images. }
var
  i: Integer;
begin
  ImageEnMView1.Clear;
  ImageEnVect1.Clear;
  ImageEnVect1.RemoveAllObjects;
  for i := 0 to StatusBar1.Panels.Count - 1 do
    StatusBar1.Panels[i].Text := '';
end;

procedure TForm1.DeleteSelectedFrame1Click(Sender: TObject);
var
  i: Integer;
begin
  if ImageEnMView1.SelectedImage <> -1 then
  begin
    ImageEnMView1.DeleteImage(ImageEnMView1.SelectedImage);
    ImageEnMView1.Update;
    ImageEnVect1.Clear;
    ImageEnVect1.RemoveAllObjects;
    if ImageEnMView1.ImageCount = 0 then
      for i := 0 to StatusBar1.Panels.Count - 1 do
        StatusBar1.Panels[i].Text := '';
  end;
end;

procedure TForm1.Exit1Click(Sender: TObject);
begin
  Close;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  IERegisterFormats;
  OpenPictureDialog1.Filter := GraphicFilter(TGraphic);
  SavePictureDialog1.Filter := GraphicFilter(TGraphic);
  ImageEnMView1.SetModernStyling(True, 150, 110);
  ImageEnVect1.ObjBitmapHandleTransparencyOnSel := True;
  ImageEnMView1.AnnotationsVisible := True;
  ImageEnVect1.Bitmap.Modified := False;
  UpdateGUI;
end;

procedure TForm1.ImageEnMView1ImageSelect(Sender: TObject; idx: Integer);
{ Display the selected image in ImageEnVect. }
begin
  if ImageEnMView1.SelectedImage > -1 then
  begin
    { Clear }
    ImageEnVect1.RemoveAllObjects;
    ImageEnVect1.Clear;

    { Copy the selected IEBitmap to the background of ImageEnVect }
    ImageEnMView1.CopyToIEBitmap(idx, ImageEnVect1.IEBitmap);

    { Copy the annotations from ImageEnMView to ImageEnVect }
    if ImageEnMView1.MIO.Params[idx].ImageEnAnnot.IsEmpty = False then
    begin
      ImageEnMView1.MIO.Params[idx].ImageEnAnnot.CopyToTImageEnVect
        (ImageEnVect1);
      { Display the object count in the statusbar.  If ImagingAnnot.ObjectsCount
        is 0 then get the object count from ImageEnVect }
      if ImageEnMView1.MIO.Params[idx].ImagingAnnot.ObjectsCount <> 0 then
        StatusBar1.Panels[5].Text := 'Objects: ' +
          IntToStr(ImageEnMView1.MIO.Params[idx].ImagingAnnot.ObjectsCount)
      else if ImageEnMView1.MIO.Params[idx].ImagingAnnot.ObjectsCount = 0 then
      begin
        MessageDlg
          (PChar('ImageEnMView1.MIO.Params[idx].ImagingAnnot.ObjectsCount is 0. '
          + 'Getting the object count from ImageEnVect instead.'), mtWarning,
          [mbOK], 0);
        StatusBar1.Panels[5].Text := 'Objects: ' +
          IntToStr(ImageEnVect1.ObjectsCount);
      end;
    end;

    { Refresh the TImageEnVect }
    ImageEnVect1.Update;
    ImageEnVect1.Fit;
    UpdateGUI;

    StatusBar1.Panels[3].Text := 'Frame: ' +
      IntToStr(ImageEnMView1.SelectedImage + 1);
  end;
end;

procedure TForm1.ImageEnVect1AfterDrawObject(Sender: TObject; hobj: Integer;
  destination: TIEBitmap; destCanvas: TIECanvas; destRect: TRect;
  drawingAlpha: Boolean; var handled: Boolean);
{ After drawing the object save the annotations and update the buttons. }
begin
  ImageEnVect1.Bitmap.Modified := True;
  SaveAnnotationsToImageEnMView;
  UpdateGUI;
end;

procedure TForm1.ImageEnVect1KeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
{ Delete the selected objects. }
var
  i: Integer;
  ihobj: Integer;
begin
  if Key = VK_DELETE then
  begin
    { Get the selected objects }
    for i := ImageEnVect1.ObjectsCount - 1 downto 0 do
    begin
      ihobj := ImageEnVect1.GetObjFromIndex(i);
      if ImageEnVect1.IsSelObject(ihobj) then
        { Remove selected object }
        ImageEnVect1.RemoveObject(ihobj);
      UpdateGUI;
    end;
  end;
end;

procedure TForm1.ImageEnVect1MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if ImageEnMView1.SelectedImage <> -1 then
    if ImageEnMView1.MIO.Params[ImageEnMView1.SelectedImage]
      .ImagingAnnot.ObjectsCount <> 0 then
      StatusBar1.Panels[5].Text := 'Objects: ' +
        IntToStr(ImageEnMView1.MIO.Params[ImageEnMView1.SelectedImage]
        .ImagingAnnot.ObjectsCount)
    else
      StatusBar1.Panels[5].Text := 'Objects: ' +
        IntToStr(ImageEnVect1.ObjectsCount);
  UpdateGUI;
end;

procedure TForm1.ImageEnVect1NewObject(Sender: TObject; hobj: Integer);
{ Add a yellow solid ellipse. }
begin
  ImageEnVect1.ObjBrushColor[hobj] := clYellow;
  ImageEnVect1.ObjBrushStyle[hobj] := bsSolid;
  StatusBar1.Panels[5].Text := 'Objects: ' +
    IntToStr(ImageEnVect1.ObjectsCount);
  StatusBar1.Panels[6].Text := 'Object: ' + IntToStr(hobj + 1);
end;

procedure TForm1.ImageEnVect1UserSelectObject(Sender: TObject; hobj: Integer);
{ Display information in statusbar. }
begin
  if ImageEnMView1.MIO.Params[ImageEnMView1.SelectedImage]
    .ImagingAnnot.ObjectsCount <> 0 then
    StatusBar1.Panels[5].Text := 'Objects: ' +
      IntToStr(ImageEnMView1.MIO.Params[ImageEnMView1.SelectedImage]
      .ImagingAnnot.ObjectsCount)
  else
    StatusBar1.Panels[5].Text := 'Objects: ' +
      IntToStr(ImageEnVect1.ObjectsCount);
  StatusBar1.Panels[6].Text := 'Object: ' + IntToStr(hobj + 1);
end;

procedure TForm1.Open1Click(Sender: TObject);
{ Open multiframe tif file with annotations. }
begin
  OpenPictureDialog1.Options := [ofHideReadOnly, ofEnableSizing];
  if OpenPictureDialog1.Execute then
  begin
    ImageEnMView1.LoadFromFileOnDemand(OpenPictureDialog1.FileName, False);
    ImageEnMView1.SelectedImage := 0;

    ImageEnVect1.IO.LoadFromFile(OpenPictureDialog1.FileName);
    ImageEnVect1.LoadObjectsFromTIFF(OpenPictureDialog1.FileName, 0);
    ImageEnVect1.Bitmap.Modified := False;

    UpdateGUI;

    StatusBar1.Panels[0].Text := ExtractFileDir(OpenPictureDialog1.FileName);
    StatusBar1.Panels[1].Text := ExtractFileName(OpenPictureDialog1.FileName);
    StatusBar1.Panels[2].Text := 'Frames: ' +
      IntToStr(ImageEnMView1.ImageCount);
    StatusBar1.Panels[3].Text := 'Frame: ' +
      IntToStr(ImageEnMView1.SelectedImage);
    if ImageEnMView1.MIO.Params[ImageEnMView1.SelectedImage]
      .ImagingAnnot.ObjectsCount <> 0 then
      StatusBar1.Panels[5].Text := 'Objects: ' +
        IntToStr(ImageEnMView1.MIO.Params[ImageEnMView1.SelectedImage]
        .ImagingAnnot.ObjectsCount)
    else
      StatusBar1.Panels[5].Text := 'Objects: ' +
        IntToStr(ImageEnVect1.ObjectsCount);
    StatusBar1.Panels[6].Text := 'Object:';
  end;
end;

procedure TForm1.SaveAs1Click(Sender: TObject);
{ Save all the tif file frames and all the annotations to disk. }
begin
  if SavePictureDialog1.Execute then
  begin
    ImageEnMView1.MIO.SaveToFileTIFF(SavePictureDialog1.FileName, False);

    ImageEnVect1.Bitmap.Modified := False;
    UpdateGUI;

    Caption := SavePictureDialog1.FileName +
      '- Load Multiframe TIF File On Demand With Annotations';
    StatusBar1.Panels[0].Text := ExtractFileDir(SavePictureDialog1.FileName);
    StatusBar1.Panels[1].Text := ExtractFileName(SavePictureDialog1.FileName);
  end;
end;

procedure TForm1.SaveAnnotations1Click(Sender: TObject);
{ Display the annotations in ImageEnMView }
begin
  SaveAnnotationsToImageEnMView;
end;

procedure TForm1.SelectObjects1Click(Sender: TObject);
{ Set the MouseInteractVt to object selection mode. }
begin
  if SelectObjects1.Down then
    ImageEnVect1.MouseInteractVt := [miObjectSelect]
  else
    ImageEnVect1.MouseInteractVt := [];
end;

end.
